@@ -24,10 +24,7 @@
 
 #include <plist/Node.h>
 #include <ctime>
-
-#ifdef _WIN32
-#include <Winsock2.h>
-#endif
+#include <sys/time.h>
 
 namespace PList
 {
